﻿using UnityEngine;
using System.Collections;

public class SnowBall : MonoBehaviour {

	public float idleVelocityThreshold; // consider idle when vel is under this value
	public float gravityScale;
	public float dragClamp = 5;

	public AudioClip shootSound;
	public AudioClip hitSound;
	public AudioClip endSound;

	//private bool touchingSnow;

	private bool shouldShoot;
	private LineRenderer lineRenderer;
	private Vector2 dragVector;
	private Vector2 forceVector;

	// Use this for initialization
	void Start () {
		lineRenderer = GetComponent<LineRenderer>();
	}
	
	// Update is called once per frame
	void Update () {

		// End game if ball is lost
		if(transform.position.y < -9) GameFlow.instance.setGameOver();

		// Calculate shoot vector and draw aiming line
		if(shouldShoot){
			lineRenderer.SetVertexCount(2);
			// ball position
			Vector2 launchPosition = new Vector2(transform.position.x, transform.position.y);
			// mouse position
			Vector3 mousePos = Camera.main.ScreenToWorldPoint(new Vector3 (Input.mousePosition.x,Input.mousePosition.y,0));
			dragVector = new Vector2(mousePos.x, mousePos.y);


			// shoot vector
			forceVector = launchPosition - dragVector;
			forceVector = Vector2.ClampMagnitude(forceVector,dragClamp);

			lineRenderer.SetPosition(0, transform.position);
			Vector2 drawVector = new Vector2(-forceVector.x, -forceVector.y);
			lineRenderer.SetPosition(1, new Vector3(drawVector.x + transform.position.x, drawVector.y + transform.position.y, 0));
		} else {
			lineRenderer.SetVertexCount(0);
		}

	}

	void OnGUI(){
		//GUI.Label(new Rect(0,0,200, 200), "Velocity: " + rigidbody2D.velocity.magnitude);
	}

	void OnCollisionEnter2D(Collision2D coll){
		if(coll.transform.tag == "Endzone"){

			// place ball in endzone
			if(name == "SnowBall1"){
				rigidbody2D.isKinematic = true;
				transform.position = new Vector3(GameFlow.instance.endZone.position.x, GameFlow.instance.endZone.position.y - 1.5f + (transform.localScale.y / 2), 0);
				GameFlow.instance.nextBall();
				name = name + "END";
				AudioSource.PlayClipAtPoint(endSound, transform.position);
			}
		}
		else if(name == "SnowBall2" && coll.transform.name == "SnowBall1END"){
			transform.position = new Vector3(GameFlow.instance.endZone.position.x, coll.transform.position.y + (coll.transform.localScale.y / 2) + (transform.localScale.y / 2), 0);
			GameFlow.instance.nextBall();
			rigidbody2D.isKinematic = true;
			name = name + "END";
			AudioSource.PlayClipAtPoint(endSound, transform.position);
		}
		else if(name == "SnowBall3" && coll.transform.name == "SnowBall2END"){
			transform.position = new Vector3(GameFlow.instance.endZone.position.x, coll.transform.position.y + (coll.transform.localScale.y / 2) + (transform.localScale.y / 2), 0);
			GameFlow.instance.finalBall();
			rigidbody2D.isKinematic = true;
			name = name + "END";
			AudioSource.PlayClipAtPoint(endSound, transform.position);
		} else if( coll.transform.tag == "Ground") {
			AudioSource.PlayClipAtPoint(hitSound, transform.position, 0.25f);
		}
	}

	void OnCollisionStay2D(Collision2D coll){
		if(isMoving()){
			//transform.localScale = transform.localScale * 1.005f; // rate of growth should depend on velocity
			transform.localScale = transform.localScale * (1 + (rigidbody2D.velocity.magnitude/850));
		}
	}

	void OnMouseDown(){
		if(!isMoving() && !rigidbody2D.isKinematic){
			shouldShoot = true;
		}
	}

	void OnMouseUp(){
		
		// Fire!
		if(shouldShoot){
			rigidbody2D.gravityScale = gravityScale;
			rigidbody2D.AddForce(forceVector, ForceMode2D.Impulse);
			shouldShoot = false;
			GameFlow.instance.strokes++;

			AudioSource.PlayClipAtPoint(shootSound, transform.position, 0.5f);
		}
	}

	void OnDrawGizmos() {
		Gizmos.color = Color.red;
		Gizmos.DrawLine(transform.position, new Vector3(transform.position.x + rigidbody2D.velocity.x, transform.position.y + rigidbody2D.velocity.y, 0));
	}

	private bool isMoving(){
		return rigidbody2D.velocity.magnitude > idleVelocityThreshold;
	}
}
